clear;
% Problema el·líptic de Dirichlet unidimensional: 
%
%                   -u_{xx}=f(x)   amb x\in[0,1],
%                   u(0)=0        condició de contorn
%                   u(1)=0        condició de contorn

%--------------------------Dades del problema------------------------------

%{
Sabem que la solució és u(x)=x^2(1-x) i fent els càlculs 
corresponents trobem que f(x)=-2+6x. Així doncs farem la
comparativa entre la solució exacta i la solució obtinguda amb FEM
%}


%--------------------Establm una malla al domini-------------------------
I=5;       
noe=I+1;    %nombre d'elements
non=noe+1;  %nombre de nodes
L=1;        %llargada del domini
h=L/(I+1);    %llargada de cada element


%-----------Construcció de la matriu de coeficients K---------------------- 

K=zeros(I,I);   %Inicialitzem la matriu

%Cas on la llargada dels elements és una constant=h
for i=1:1:I-1
    K(i,i)=2;
    K(i,i+1)=-1;
    K(i+1,i)=-1;
end
K(I,I)=2;

K=(1/h)*K;

%display(K);

%-------------Càlcul del vector de termes independents F ------------------

F=zeros(I,1);
for i=1:1:I
    F(i,1)=-2+6*i*h;
end
F=h*F;
%disp(F); 


%--------------------Solució del sistema lineal----------------------------
U=K\F;

%disp(U);

%--------------Afegim al vector solució les condicions de contorn----------
U_sol(1)=0;
U_sol(I+2)=0;
for i=1:1:I
    U_sol(i+1)=U(i);
end

disp(U_sol);


%--------------Grafiquem ambdues solucions: exacta i FEM-------------------
syms t x
u=@(x) x*x*(1-x);

figure(1);
hold on;
title('Exacta vs FEM')

fplot(u(x),[0 1]);

hold on 
x=0:noe; 
x=x*h;    %Canvi d'escala, volem que entre x(i) i x(i+1) hi hagi un espai h
axis([0 1 0 0.15]);

plot(x,U_sol)

grid
xlabel('x')
ylabel('u')
legend('Exacta','FEM')

hold off



%--------------Calculem l'error màxim--------------------------------------
%Interpolem la funció U_sol a una funció poligonal P
for i=0:1:noe
    interv(i+1)=i*h;
end
%disp(interv);
P = griddedInterpolant(interv,U_sol);

%Calculem el màxim de la diferència entre P i U exacte, avaluant cada 0.01
errormax=0;
for i=0:0.01:1
    error=abs(P(i)-u(i));
    if error>errormax
        errormax=error;
        errormaxrel=errormax/u(i);
        index=i;
    end
end
disp('Error=');
disp(errormax);
disp('Error relatiu=');
disp(errormaxrel*100);
disp('Punt error=');
disp(index);







